//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.c
//  Version: 1.0, 2007/03/08
//  Generated by Andy Kiser
//
//  DESCRIPTION:  sensor specific functions 
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__
#include <m8c.h>        // part specific constants and macros
#include "sensor.h"
#include "psocgpioint.h"
#include "spim.h"
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"
#include "pga_1.h"
#include "pga_2.h"
#include "i2cm_1.h"
#include "Table_TC_K.h"
#include "Table_TC_J.h"
#include "Table_TC_T.h"
//#include "Table_TC_R.h"

//----------------------------------------
// Sensor parameters
// Factory Defaults are defined here (NUM_xxx are defined in "sensor.h")
//----------------------------------------
const BYTE FactoryDefault_Param_8   [NUM_PARAM_8] =
{
	0x00,		// report mode
	0x00,		// LED mode
	0x00,		// Sensor 1 Type
	0x00,		// Sensor 2 Type
};

const WORD FactoryDefault_Param_16  [NUM_PARAM_16] = 
{
	500,		// report rate [ms]
	500,		// sample rate [ms]
	50000,		// sample rate [us]	(not used for this sensor)
};

const DWORD FactoryDefault_Param_32  [NUM_PARAM_32] =
{
	0x00000000	// currently no 32bit parameters
};

const tParam_168 FactoryDefault_Param_168 [NUM_PARAM_168] =
{
	"Thermocouple Sensor",	// user device name
	"Frame Number",		// user name Channel 0
	"Thermocouple 1",	// user name Channel 1
	"Thermocouple 2",	// user name Channel 1
	"Cold Junction",	// user name Channel 1
};

const BYTE I2C_Settings[2] = {0x04,0x68};	//14Bit
//const BYTE I2C_Settings[2] = {0x04,0x08};	//11Bit
const BYTE I2C_SetPointer = 0x00;


//----------------------------------------
// this module's global variables
//----------------------------------------
tReport Report;
extern BYTE SOF_Flag;	// Set in SOF_ISR


//----------------------------------------
// type definitions
//----------------------------------------

typedef enum 
{	
	eReset,
	eIdle,
	eMeasuring,
	eReadADC0,
	eReadADC1
} tFSM_Measure;

//----------------------------------------
// variables

tFSM_Measure	FSM;
WORD			Countdown_1ms;

BOOL			NewSample = FALSE;

// store original drivemodes during suspend
BYTE			DM0_2, DM0_1, DM0_0;
BYTE			DM2_2, DM2_1, DM2_0;

//typedef unsigned int cUint;
//typedef const cUint pcUint;
//typedef const unsigned int* pcUint;

int ADC2Tmp_Min_1;
int ADC2Tmp_Mid_1;
int ADC2Tmp_Max_1;

const unsigned int *ADC2TmpLo_1;
const unsigned int *ADC2TmpHi_1;
const int *Ref2ADC_1;

//int ADC2Tmp_Min_2;
//int ADC2Tmp_Mid_2;
//int ADC2Tmp_Max_2;

//const unsigned int *ADC2TmpLo_2;
//const unsigned int *ADC2TmpHi_2;
//const int *Ref2ADC_2;

//----------------------------------------
// function prototypes
//----------------------------------------
void SetType(BYTE Channel);


//-----------------------------------------------------------------------------
// Initializes the Sensor 
//-----------------------------------------------------------------------------
void Sensor_Start ()
{
	FSM = eReset;

	// SCK idle state = 1, to conserve power
	// ATTENTION: during each falling edge of nCS, SCK *must* be *low*, to 
	//            activate external SCK mode !!!
	SPIM_Start(SPIM_SPIM_MODE_1 | SPIM_SPIM_MSB_FIRST);
	// nCS idle state = 1
	GPIO_HIGH(nCS)
	
	// turn on power for the sensor
	GPIO_HIGH(nOFF_ON)
	
	// turn on 3.3V Output for temperature Sensor
	PGA_1_Start(PGA_1_HIGHPOWER);
	// turn on the I2C interface;
	I2Cm_1_Start();
	// Configure the temperature sensor
	//I2Cm_1SCL_Pin_Data_ADDR &= 0xD7;
	//I2Cm_1_bWriteCBytes(0x48,&I2C_Settings[0],2,I2Cm_1_CompleteXfer);	//Set to 14 Bit mode
	//I2Cm_1_bWriteCBytes(0x48,&I2C_SetPointer,1,I2Cm_1_CompleteXfer);	//Set pointer to 0 for reading the temperature
	
	// Set the type of the thermocouples
	//SetType(); 
	Countdown_1ms = 500; //Test
}
//-----------------------------------------------------------------------------
// Prepare Sensor for Suspend
//-----------------------------------------------------------------------------
void Sensor_Suspend()
{
	// turn off power for the sensor
	GPIO_LOW(nOFF_ON)
	// turn off the I2C interface;
	I2Cm_1_Stop();
	PGA_1_Stop();

	// store original drivemodes during  suspend
	DM0_2 = PRT0DM2;
	DM0_1 = PRT0DM1;
	DM0_0 = PRT0DM0;
	DM2_2 = PRT2DM2;
	DM2_1 = PRT2DM1;
	DM2_0 = PRT2DM0;
	
	// set all I/Os to/from the sensor to High-Z Analog (=110), 
	// except the OFF_nON signal. 
	// OFF_nON is on Port0, all other signals on Port2
	PRT0DM2 |= ~nOFF_ON_MASK;
	PRT0DM1 |= ~nOFF_ON_MASK;
	PRT0DM0 &=  nOFF_ON_MASK;
	PRT2DM2  = 0xFF;
	PRT2DM1  = 0xFF;
	PRT2DM0  = 0x00;

}
//-----------------------------------------------------------------------------
// Resume Sensor from Suspend mode
//-----------------------------------------------------------------------------
void Sensor_Resume()
{
	GPIO_HIGH(nOFF_ON)
	
	// restore drivemodes for GPIOs to/from the sensor
	PRT2DM2 = DM2_2;
	PRT2DM1 = DM2_1;
	PRT2DM0 = DM2_0;

	PRT0DM2 = DM0_2;
	PRT0DM1 = DM0_1;
	PRT0DM0 = DM0_0;

	// start up sensor again
	Sensor_Start();
}

//-----------------------------------------------------------------------------
// Read Raw ADC value from the sensor
// The function assumes the following (idle) conditions on entry:
// 		CSn == 0
//		SCK == 0
//		MISO== 0 (indicating that conversion has finished)
// 		SPI-Mode == 0
// Side Effect
//-----------------------------------------------------------------------------
void Sensor_ReadADC (WORD *ADCRaw, BYTE *Status)
{
	signed char i;
	unsigned long test;
	union
	{
		unsigned long ul;
		BYTE b[4];
	} Rx;
	
	// -- dummy read to clear RX_BUFFER_FULL flag
	SPIM_bReadRxData(); 
	
	// -- receive 3+16+5 bits from the Sensor
	for (i=1; i<=3; i++)
	{		// read one byte
		SPIM_SendTxData( 0x00);
		while( !( SPIM_bReadStatus() & SPIM_SPIM_RX_BUFFER_FULL ) );   	
		Rx.b[i] = SPIM_bReadRxData();
	}
	Rx.b[0] = 0;

	// -- prepare return values
	Rx.ul <<= 3;
		// 3 Bit Status
	test = Rx.ul;
	*Status     = Rx.b[0];
		// 16 bit ADC value
	*ADCRaw = 256*Rx.b[1] + Rx.b[2];
}

//-----------------------------------------------------------------------------
// Calculates the Temperature out of the ADC
//-----------------------------------------------------------------------------
WORD ADC2Tmp(WORD ADCRaw)
{
	union
	{
		BYTE			b[2];
		WORD			w;
		int			 	i;
		unsigned int	ui;
	} Res,tmp1;
	union
	{
		BYTE			b[4];
		WORD			w[2];
		long		 	l;
		unsigned long	ul;
	} tmp2;
	Res.w = ADCRaw;
	if(((int)ADCRaw<ADC2Tmp_Min_1)||((int)ADCRaw>ADC2Tmp_Max_1))	//Check if voltage is out of range
	{
		Res.w = 0xFFFF;
	}
	else if ((int)ADCRaw<ADC2Tmp_Mid_1)	//Lower Table
	{
		Res.i -= ADC2Tmp_Min_1;
		Res.w <<=2;
		tmp1.ui = ADC2TmpLo_1[Res.b[0]+1] - ADC2TmpLo_1[Res.b[0]];
		tmp2.ul = (unsigned long)tmp1.ui * Res.b[1];
		tmp2.ul += 128;		// Round the interpolated part
		Res.w = ADC2TmpLo_1[Res.b[0]] + tmp2.b[2]+tmp2.b[1]*256;
	}
	else	//Higher Table
	{
		Res.i -= ADC2Tmp_Mid_1;
		tmp1.ui = ADC2TmpHi_1[Res.b[0]+1] - ADC2TmpHi_1[Res.b[0]];
		tmp2.ul = (unsigned long)tmp1.ui * Res.b[1];
		tmp2.ul += 128;		// Round the interpolated part
		Res.w = ADC2TmpHi_1[Res.b[0]] + tmp2.b[2]+tmp2.b[1]*256;
	}
	return Res.w;
}	


//-----------------------------------------------------------------------------
// Calculates the according ADC value of the cold junction temperature
//-----------------------------------------------------------------------------
void CJTmp2ADC(WORD CJTmp, WORD *CJADC1)
{
	union
	{
		BYTE			b[2];
		WORD			w;
		int			 	i;
		unsigned int	ui;
	} Res,tmp1;

	Res.w = CJTmp;
	//if(((int)Res.i < REF2ADC_MIN_R)||((int)Res.i > REF2ADC_MAX_R))	//Check if voltage is out of range
	//{
	//	Res.w = 0x6FFF;
	//}
	//else	
	//{
		Res.i -= REF2ADC_MIN_K;
		Res.i >>=1;
		tmp1.ui = Ref2ADC_1[Res.b[0]+1] - Ref2ADC_1[Res.b[0]];
		tmp1.ui = tmp1.ui * Res.b[1];
		tmp1.ui += 128;		// Round the interpolated part
		*CJADC1 = Ref2ADC_1[Res.b[0]] + tmp1.b[0];
	//}
}	

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------
BYTE Sensor_Specific_Feature(void)
{
	if  ( (!FEATURE_RPT.OUT.Get_nSet) && (FEATURE_RPT.OUT.Target == eAction) &&
	      (FEATURE_RPT.OUT.ParamSize == 0) && (FEATURE_RPT.OUT.Index_LSB == 0x80) )
		ABF_CR0 |= 0x20;
		PGA_2_Start(PGA_2_HIGHPOWER);
	return 0;
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets / gets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes)
{
	NumberOfReturnedBytes = 0;
}

//-----------------------------------------------------------------------------
// Sets the correct Tables for the according thermocouple types
//-----------------------------------------------------------------------------
void SetType(BYTE Channel)
{	
	BYTE Type;
	if (Channel == 0)
		Get_Param_8(PAR8_SENS_1_TYPE, VOLATILE, &Type);
	else
		Get_Param_8(PAR8_SENS_2_TYPE, VOLATILE, &Type);
	switch(Type)
	{
		case TC_TYPE_K:
		case TC_TYPE_K_ASCII:
		default:
			ADC2Tmp_Min_1 	= ADC2TMP_MIN_K;
			ADC2Tmp_Mid_1 	= ADC2TMP_MID_K;
			ADC2Tmp_Max_1 	= ADC2TMP_MAX_K;
			ADC2TmpLo_1 	=   ADC2TmpLo_K;
			ADC2TmpHi_1		=   ADC2TmpHi_K;
			Ref2ADC_1 		=     Ref2ADC_K;
			break;
		case TC_TYPE_J:
		case TC_TYPE_J_ASCII:
			ADC2Tmp_Min_1 	= ADC2TMP_MIN_J;
			ADC2Tmp_Mid_1 	= ADC2TMP_MID_J;
			ADC2Tmp_Max_1 	= ADC2TMP_MAX_J;
			ADC2TmpLo_1 	=   ADC2TmpLo_J;
			ADC2TmpHi_1		=   ADC2TmpHi_J;
			Ref2ADC_1 		=     Ref2ADC_J;
			break;
		case TC_TYPE_T:
		case TC_TYPE_T_ASCII:
			ADC2Tmp_Min_1 	= ADC2TMP_MIN_T;
			ADC2Tmp_Mid_1 	= ADC2TMP_MID_T;
			ADC2Tmp_Max_1 	= ADC2TMP_MAX_T;
			ADC2TmpLo_1 	=   ADC2TmpLo_T;
			ADC2TmpHi_1		=   ADC2TmpHi_T;
			Ref2ADC_1 		=     Ref2ADC_T;
			break; 
/*		case TC_TYPE_R:
		case TC_TYPE_R_ASCII:
			ADC2Tmp_Min_1 	= ADC2TMP_MIN_R;
			ADC2Tmp_Mid_1 	= ADC2TMP_MID_R;
			ADC2Tmp_Max_1 	= ADC2TMP_MAX_R;
			ADC2TmpLo_1 	=   ADC2TmpLo_R;
			ADC2TmpHi_1		=   ADC2TmpHi_R;
			Ref2ADC_1 		=     Ref2ADC_R;
			break;*/
	}
}


//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the Sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
void Sensor_FSM_Measure() //ADC read Algorithm has to be changed!!!!!!!!!!!!!!!!!!!!!!!!!!!
{
	//WORD ADC;
	BYTE Status;
	char i;
	
	union
	{
		BYTE			b[4];
		WORD			w[2];
		int				i[2];
		unsigned long 	ul;
	} ADCRaw;
	static union
	{
		BYTE			b[2];
		int				i;
		WORD			w;
	} CJTemp,CJADC,CJTempRaw;

	BYTE ElapsedTime = SOF_Flag;
	SOF_Flag = 0;	//Has to be done directly after reading the Elapsed Time
	// decrement millisecond counter
	if (Countdown_1ms >= ElapsedTime)
		Countdown_1ms -= ElapsedTime;
	else if(Countdown_1ms >0)
		Countdown_1ms = 0;

	// process actual state machine
	switch (FSM)
	{
		case eReset:
			//GPIO_HIGH(nOFF_ON)

			// ADC automatically starts conversion on Power-on Reset (POR)
			I2Cm_1SCL_Pin_Data_ADDR &= 0xD7;
			I2Cm_1_bWriteCBytes(0x48,&I2C_Settings[0],2,I2Cm_1_CompleteXfer);	//Set to 14 Bit mode
			I2Cm_1_bWriteCBytes(0x48,&I2C_SetPointer,1,I2Cm_1_CompleteXfer);	//Set pointer to 0 for reading the temperature
			FSM = eMeasuring;
			break;
			
		case eMeasuring:
			if (Countdown_1ms == 302)
			{
				//starting the cold junction temperature sensor to sample
				I2Cm_1SCL_Pin_Data_ADDR &= 0xD7;
				//I2Cm_1_bWriteCBytes(0x48,&I2C_Settings[0],2,I2Cm_1_CompleteXfer);	//Set to 14 Bit mode
				//I2Cm_1_bWriteCBytes(0x48,&I2C_SetPointer,1,I2Cm_1_CompleteXfer);	//Set pointer to 0 for reading the temperature
				I2Cm_1_fReadBytes(0x48,&CJTempRaw.b[0],2,I2Cm_1_CompleteXfer);	//Dummy read
				// Activate SPI and bring SCK signal to low
				SPIM_Start(SPIM_SPIM_MODE_0 | SPIM_SPIM_MSB_FIRST);
				
				// activate nCS to be able to detect EOC flag
				GPIO_LOW(nCS)  
				for(i=0; i<10;i++);  // short delay to allow signal to settle
				if ( (MISO_Data_ADDR & MISO_MASK) == 0 )
				{	
					// Countdown_1ms == 151, enough time to do one more conversion, before result is requested
					// => generate at least one clock cycle
					SPIM_bReadRxData(); 	// dummy read to clear RX_BUFFER_FULL flag
					SPIM_SendTxData( 0x00);
					while( !( SPIM_bReadStatus() & SPIM_SPIM_RX_BUFFER_FULL ) );   	
				}
				// deactivate nCS to start a new conversion
				GPIO_HIGH(nCS)
				SPIM_Start(SPIM_SPIM_MODE_1 | SPIM_SPIM_MSB_FIRST);

			}
			if (Countdown_1ms <= 302)
				FSM = eReadADC0;
			break;

		case eReadADC0:
			if (Countdown_1ms == 151)
			{
				//reading the cold junction temperature sensor
				I2Cm_1SCL_Pin_Data_ADDR &= 0xD7;
				I2Cm_1_fReadBytes(0x48,&CJTempRaw.b[0],2,I2Cm_1_CompleteXfer);
				//Calculate the coldjunction temperature for the output (*10/128)
				CJTemp.i = (CJTempRaw.i+(CJTempRaw.i>>2));
				CJTemp.i >>= 4;
				CJTemp.i += 2732;
				Report.Ch3_LSB = CJTemp.b[1];
				Report.Ch3_MSB = CJTemp.b[0];

				// Activate SPI and bring SCK signal to low
				SPIM_Start(SPIM_SPIM_MODE_0 | SPIM_SPIM_MSB_FIRST);
			
				GPIO_LOW(nCS)
				for(i=0; i<10;i++);  // short delay to allow signal to settle
				if ( (MISO_Data_ADDR & MISO_MASK) == 0 )
				{
					// reset samplerate counter
					//Countdown_1ms -=3; // offset for processing that takes longer than 1ms
	
					// reading the ADC automatically starts a new conversion.
					Sensor_ReadADC (&ADCRaw.w[0], &Status);
					if(Status&0x02)
					{
						//Calculate the according ADC Voltage of the compensation
						SetType(1);
						CJTmp2ADC(CJTempRaw.w,&CJADC.w);
						ADCRaw.w[1] = ADC2Tmp(ADCRaw.w[0] + CJADC.w);
						Report.Ch2_LSB = ADCRaw.b[3];
						Report.Ch2_MSB = ADCRaw.b[2];
					}
					else
					{
						//Calculate the according ADC Voltage of the compensation
						SetType(0);
						CJTmp2ADC(CJTempRaw.w,&CJADC.w);
						//Report.Ch3_LSB = ADCRaw.b[1];
						//Report.Ch3_MSB = ADCRaw.b[0];
						ADCRaw.w[1] = ADC2Tmp(ADCRaw.w[0] + CJADC.w);
						//ADCRaw.w[1] = ADC2TmpCH1(ADCRaw.w[0]);
						Report.Ch1_LSB = ADCRaw.b[3];
						Report.Ch1_MSB = ADCRaw.b[2];
					}
				}
				GPIO_HIGH(nCS)
				SPIM_Start(SPIM_SPIM_MODE_1 | SPIM_SPIM_MSB_FIRST); // only to bring SCK to high
			}
			if (Countdown_1ms <= 151)
				FSM = eReadADC1;
			break;
			break;			
		case eReadADC1:
			if (Countdown_1ms == 0)
			{
				// Activate SPI and bring SCK signal to low
				SPIM_Start(SPIM_SPIM_MODE_0 | SPIM_SPIM_MSB_FIRST);
			
				GPIO_LOW(nCS)
				for(i=0; i<10;i++);  // short delay to allow signal to settle
				if ( (MISO_Data_ADDR & MISO_MASK) == 0 )
				{
					// reset samplerate counter
					Get_Param_16(PAR16_SAMPLERATE_MS, VOLATILE, &Countdown_1ms);
					if (Countdown_1ms < 300) Countdown_1ms = 300;
					//Countdown_1ms -=3; // offset for processing that takes longer than 1ms
					FSM = eMeasuring;

					// reading the ADC automatically starts a new conversion.
					Sensor_ReadADC (&ADCRaw.w[0], &Status);
					if(Status&0x02)
					{
						//Calculate the according ADC Voltage of the compensation
						SetType(1);
						CJTmp2ADC(CJTempRaw.w,&CJADC.w);
						ADCRaw.w[1] = ADC2Tmp(ADCRaw.w[0] + CJADC.w);
						Report.Ch2_LSB = ADCRaw.b[3];
						Report.Ch2_MSB = ADCRaw.b[2];
					}
					else
					{
						//Calculate the according ADC Voltage of the compensation
						SetType(0);
						CJTmp2ADC(CJTempRaw.w,&CJADC.w);
						//Report.Ch3_LSB = ADCRaw.b[1];
						//Report.Ch3_MSB = ADCRaw.b[0];
						ADCRaw.w[1] = ADC2Tmp(ADCRaw.w[0] + CJADC.w);
						//ADCRaw.w[1] = ADC2TmpCH1(ADCRaw.w[0]);
						Report.Ch1_LSB = ADCRaw.b[3];
						Report.Ch1_MSB = ADCRaw.b[2];
					}
					NewSample = TRUE;
				}
				GPIO_HIGH(nCS)
				SPIM_Start(SPIM_SPIM_MODE_1 | SPIM_SPIM_MSB_FIRST); // only to bring SCK to high
			}
			break;			
	}
}

//-----------------------------------------------------------------------------
void Sensor_Debug(void)
{
/*			ADC2Tmp_Min_1 	= ADC2TMP_MIN_J;
			ADC2Tmp_Mid_1 	= ADC2TMP_MID_J;
			ADC2Tmp_Max_1 	= ADC2TMP_MAX_J;
			ADC2TmpLo_1 	=   ADC2TmpLo_J;
			ADC2TmpHi_1		=   ADC2TmpHi_J;
			Ref2ADC_1 		=     Ref2ADC_J;*/
/* currently no debug function */
}